

% This function is called by the main Matlab program for 
% running imaging polarimetric self-calibration.
% This function estimates the polarizer (analyser) angles, and
% P-matrix, based on assumed set of object Stokes vectors. The function 
% also returns the MSE of this estimation.
%
% You are very welcome to use this code. For this, clearly acknowledge 
% the source of this code, and cite the paper that describes the algorithm:
% Y. Y. Schechner, "Self-Calibrating Imaging Polarimetry", 
% Proc. IEEE Int. Conference on Computational Photography (ICCP), 2015.
%
% (c) Copyright Yoav Schechner 2015. The matlab code is available for 
% non-commercial use and exploration.  For commercial use contact the
% author. The author is not liable for any damages or loss that might be 
% caused by use or connection to this code. 


function [p_angle_degrees,polarizance,hatP,MSEmat]=optimizeP_givenO(subdata,Object_matrix)

[tildeOneto,inversetildeOneto,Ineto_transpos]=object_matrices(subdata,Object_matrix);

hat_tildeP_neto=inversetildeOneto*Ineto_transpos;

num_of_pixels=size(subdata,2);
num_of_angles=size(subdata,1);

Pneto_transpose_matrix=reshape(hat_tildeP_neto,2,(num_of_angles-1));

pc=Pneto_transpose_matrix(1,:);
ps=Pneto_transpose_matrix(2,:);
p_vector=pc+i*ps;
p_mag=abs(p_vector);

polarizance=[1 p_mag];

p_angle=angle(p_vector)/2;
p_angle_degrees=[0 mod(radtodeg(p_angle),180)];

pc=pc./p_mag;
ps=ps./p_mag;


Pneto=[pc' ps'];
Pwith0=[1 0 ;Pneto];

hatP=[ones(num_of_angles,1), Pwith0];


Emat=(abs(hatP*Object_matrix-subdata)).^2;
MSEmat=mean(Emat(:));
